/*******************************************************************************

    FREQ_DETECT

    C/ PIC 12F675

    DETECTA FREQUNCIAS DENTRO DE UMA FAIXA DE TRABALHO E ACIONA SADA.


    AUTOR: CLAUDIO LARIOS

    DATA:08/09/2020



    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>//pic usado
#device adc=10
#use delay(clock=4000000)//frequ de clock
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, MCLR, PUT, BROWNOUT//configurao

//bytes
#byte option_reg = 0x81
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  t1con  = 0x10
#byte  intcon = 0x0b
#byte  cmcon  = 0x19
#byte  tmr0   = 0x01
#byte  ansel  = 0x9f
#byte  tmr1h   = 0x0f
#byte  tmr1l   = 0x0e

#bit t0ie  =    0x0b.5
#bit t0if  =    0x0b.2
#bit gie   =    0x0b.7
#bit tmr1if =   0x0c.0 //flag overflow timer 1

#bit in_freq   = 0x05.2  //pino 5 - entrada da frequncia
#bit led       = 0x05.5  //pino 2 - acende quando freq na entrada esta na faixa passante
#bit tx_out    = 0x05.4  //pino 3 - saida para rs 232 tx

//variveis globais
unsigned int8 tmr_salvo=0;
unsigned int8 cent=0,dez=0,un=0;

//defines
#define USAR_SERIAL  // Deixe descomentado para poder enviar pela SERIAL RS232


#define FREQ_MAX 33792 / 256
#define FREQ_MIN 32000 / 256


//sub-rotinas
/*******************************************************************************
     Converte valor para decimal
*******************************************************************************/
void converte_dec(unsigned int8 dado){
  cent ='0';
  dez ='0';

  while(dado>=100){
    cent++;
    dado = dado - 100;
  }
  while(dado>=10){
    dez++;
    dado = dado - 10;
  }
  un = dado |'0';

}


void env_byte(unsigned int8 bser){
  unsigned int8 a;
   tx_out = 0;
  delay_us(25); //start bit
  for(a=0; a<8; a++){
   if(bser & 1) tx_out = 1; else tx_out = 0;
   bser = bser>>1;
   delay_us(11);
  }
  tx_out = 1;
  delay_us(26); //stop bit
}





void env_ser(unsigned int8 val){
  converte_dec(val);
  env_byte('T');
  env_byte('M');
  env_byte('R');
  env_byte('=');
  env_byte(' ');
  env_byte(cent);
  env_byte(dez);
  env_byte(un);
  env_byte(' ');
  delay_ms(250);

}
/*******************************************************************************
                        ROTINA MAIN
*******************************************************************************/

void main() {
   ansel = 0;
   option_reg = 0b10101000;
   trisio = 0b001111;//ajusta entradas e sadas
   cmcon = 7;//desliga comparadores
   gpio = 0;//zera tudo



/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


  
  


   for(;;){
   // o timer 1 foi usado para dar mais preciso na janela de tempo de leitura
   // feita pelo timer 0.
   
    t1con = 0; //desliga timer 1
    tmr1h = 0xf0; // para contar 3906 us (65535 - 3906 = 61629 ou 0xf0bd)
    tmr1l = 0xbd;
    tmr1if=0; //reseta flag de overflow do timer 1
    tmr0 = 0;
    t0if = 0;
    tmr0 = 0;
    t1con=1;
    while(!tmr1if);// aguarda enquanto acaba a janela de tempo de leitura do timer 0    
    tmr_salvo = tmr0;//salva a leitura do timer 0
    #ifdef  USAR_SERIAL
    env_ser(tmr_salvo);
    #endif
    if(tmr_salvo >= FREQ_MIN && tmr_salvo <= FREQ_MAX) {
      led = 1; //frequncia na entrada esta na faixa determinada - acende led
    }
    else {
      led = 0;//frequncia esta fora da faixa determinada - mantm apagado o led
    }
    delay_ms(50);
   }//for(;;)
   
 

}//void main()


