/*******************************************************************************

    FREQ_DETECT

    C/ PIC 12F675

    DETECTA FREQUNCIAS DENTRO DE UMA FAIXA DE TRABALHO E ACIONA SADA.


    AUTOR: CLAUDIO LARIOS

    DATA:08/09/2020

    COMPILE COM MPLABX IDE V3.05

    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <xc.h>

#define _XTAL_FREQ  4000000  //xtal 4Mhz

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG
#pragma config FOSC = INTRCIO   // Oscillator Selection bits (INTOSC oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = ON      // RA3/MCLR pin function select (RA3/MCLR pin function is digital I/O, MCLR internally tied to VDD)
#pragma config BOREN = OFF      // Brown-out Detect Enable bit (BOD disabled)
#pragma config CP = OFF         // Code Protection bit (Program Memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)

#define in_freq      GPIO2  //pino 5 entrada de frequencia
#define led          GPIO5  //pino 2 led indicador de faixa de frequencia encontrada
#define tx_out_      GPIO4  //pino 3 - saida para rs 232 tx

//variveis globais
char tmr_salvo=0;
char cent=0,dez=0,un=0;

//defines
#define USAR_SERIAL  // Deixe descomentado para poder enviar pela SERIAL RS232


#define FREQ_MAX 33792 / 256
#define FREQ_MIN 32000 / 256


//sub-rotinas
/*******************************************************************************
     Converte valor para decimal
*******************************************************************************/
void converte_dec(char dado1){
  cent ='0';
  dez ='0';

  while(dado1>=100){
    cent++;
    dado1 = dado1 - 100;
  }
  while(dado1>=10){
    dez++;
    dado1 = dado1 - 10;
  }
  un = dado1 |'0';

}

/*
void env_byte(char bser){
  char a;
   tx_out_= 0;
  __delay_us(24); //start bit 25
  for(a=0; a<8; a++){
   if(bser & 1) tx_out_= 1; else tx_out_= 0;
   bser = bser>>1;
   __delay_us(1);
  }
  tx_out_= 1;
  __delay_us(24); //stop bit
}
*/
void env_byte(char bser){
    
    //rotina crtica para baud rate 38400
    //confira arquivo list para ter certeza dos tempos caso troque a verso do
    //compilador.
    tx_out_= 0;
  __delay_us(24); //start bit 25
  if(bser & 1) tx_out_= 1; else {tx_out_=0; __delay_us(1);}
   __delay_us(18); // bit0 
  if(bser & 2) tx_out_= 1; else {tx_out_=0; __delay_us(1);}
   __delay_us(18); // bit 1 
  if(bser & 4) tx_out_= 1; else {tx_out_=0; __delay_us(1);}
   __delay_us(18); // bit 2
   if(bser & 8) tx_out_= 1; else {tx_out_=0; __delay_us(1);}
   __delay_us(18); // bit 3
  if(bser & 16) tx_out_= 1; else {tx_out_=0; __delay_us(1);}
   __delay_us(18); // bit  4
  if(bser & 32) tx_out_= 1; else {tx_out_=0; __delay_us(1);}
   __delay_us(18); // bit  5 
  if(bser & 64) tx_out_= 1; else {tx_out_=0; __delay_us(1);}
   __delay_us(18); // bit  6
  if(bser & 128) tx_out_= 1; else {tx_out_=0; __delay_us(1);}
   __delay_us(18); // bit  7   
  tx_out_= 1;
  __delay_us(24); //stop bit
}

void env_ser(char val){
  converte_dec(val);
  env_byte('T');
  env_byte('M');
  env_byte('R');
  env_byte('=');
  env_byte(' ');
  env_byte(cent);
  env_byte(dez);
  env_byte(un);
  env_byte(' ');
  __delay_ms(250);

}
/*******************************************************************************
                        ROTINA MAIN
*******************************************************************************/

void main() {
   ANSEL = 0;
   OPTION_REG = 0b10101000;
   TRISIO  = 0b001111;//ajusta entradas e sadas
   CMCON = 7;//desliga comparadores
   GPIO = 0;//zera tudo



/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


  
  


   for(;;){
   // o timer 1 foi usado para dar mais preciso na janela de tempo de leitura
   // feita pelo timer 0.
   
    T1CON = 0; //desliga timer 1
    TMR1H = 0xf0; // para contar 3906 us (65535 - 3906 = 61629 ou 0xf0bd)
    TMR1L = 0xbd;
    TMR1IF=0; //reseta flag de overflow do timer 1
    TMR0 = 0;
    T0IF = 0;
    TMR0 = 0;
    T1CON=1;
    while(!TMR1IF);// aguarda enquanto acaba a janela de tempo de leitura do timer 0    
    tmr_salvo = TMR0;//salva a leitura do timer 0
    #ifdef  USAR_SERIAL
    env_ser(tmr_salvo);
    #endif
    if(tmr_salvo >= FREQ_MIN && tmr_salvo <= FREQ_MAX) {
      led = 1; //frequncia na entrada esta na faixa determinada - acende led
    }
    else {
      led = 0;//frequncia esta fora da faixa determinada - mantm apagado o led
    }
    __delay_ms(50);
   }//for(;;)
   
 

}//void main()
